"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.make = exports.addStatements = exports.update = exports.createMultiple = exports.create = exports.findStatement = void 0;
const typescript_1 = __importDefault(require("typescript"));
const ModuleBlock = __importStar(require("./ModuleBlock"));
const utils_1 = require("./utils");
// eslint-disable-next-line no-unused-vars
const findStatement = (_context) => (params) => {
    let statement;
    params.node.forEachChild(node => {
        if (typescript_1.default.isModuleDeclaration(node) && node.name.text === params.name) {
            statement = node;
        }
    });
    return statement;
};
exports.findStatement = findStatement;
const create = ({ factory }) => (params) => {
    const node = factory.createModuleDeclaration(params.export && [factory.createModifier(typescript_1.default.SyntaxKind.ExportKeyword)], factory.createIdentifier(params.name), factory.createModuleBlock(params.statements), typescript_1.default.NodeFlags.Namespace);
    if (params.comment) {
        const comment = (0, utils_1.generateComment)(params.comment, params.deprecated);
        return typescript_1.default.addSyntheticLeadingComment(node, typescript_1.default.SyntaxKind.MultiLineCommentTrivia, comment.value, comment.hasTrailingNewLine);
    }
    return node;
};
exports.create = create;
const createMultiple = (context) => (params) => {
    const names = params.names.reverse();
    const firstName = names[0];
    const restNames = names.slice(1, names.length);
    const child = (0, exports.create)(context)({
        export: true,
        name: firstName,
        statements: params.statements,
        comment: params.comment,
        deprecated: params.deprecated,
    });
    return restNames.reduce((previousStatement, currentName) => {
        return (0, exports.create)(context)({
            export: true,
            name: currentName,
            statements: [previousStatement],
        });
    }, child);
};
exports.createMultiple = createMultiple;
const update = (context) => (params) => {
    const { factory } = context;
    const { node, statements } = params;
    if (node.body && typescript_1.default.isModuleBlock(node.body)) {
        const body = ModuleBlock.update(context)({ node: node.body, statements });
        return factory.updateModuleDeclaration(node, node.decorators, node.modifiers, node.name, body);
    }
    return factory.updateModuleDeclaration(node, node.decorators, node.modifiers, node.name, node.body);
};
exports.update = update;
const addStatements = (context) => (params) => {
    const { factory } = context;
    const { node, statements } = params;
    if (node.body && typescript_1.default.isModuleBlock(node.body)) {
        const body = ModuleBlock.update(context)({ node: node.body, statements: node.body.statements.concat(statements) });
        return factory.updateModuleDeclaration(node, node.decorators, node.modifiers, node.name, body);
    }
    return factory.updateModuleDeclaration(node, node.decorators, node.modifiers, node.name, node.body);
};
exports.addStatements = addStatements;
const make = (context) => {
    return {
        findNamespace: (0, exports.findStatement)(context),
        create: (0, exports.create)(context),
        update: (0, exports.update)(context),
        createMultiple: (0, exports.createMultiple)(context),
        addStatements: (0, exports.addStatements)(context),
    };
};
exports.make = make;
//# sourceMappingURL=Namespace.js.map