import * as Utils from "../utils";
import { escapeText2 as escapeText } from "../../../utils";
export const isPathParameter = (params) => {
    return params.in === "path";
};
/**
 * const url = this.baseUrl + `[head]${params.parameter.[parameterName]}`;
 */
const generateUrlVariableStatement = (factory, urlTemplate, variableExpression) => {
    return factory.VariableStatement.create({
        declarationList: factory.VariableDeclarationList.create({
            declarations: [
                factory.VariableDeclaration.create({
                    name: "url",
                    initializer: factory.BinaryExpression.create({
                        left: variableExpression,
                        operator: "+",
                        right: Utils.generateTemplateExpression(factory, urlTemplate),
                    }),
                }),
            ],
            flag: "const",
        }),
    });
};
/**
 * const uri = `[head]${params.parameter.[parameterName]}`;
 */
const generateUriVariableStatement = (factory, urlTemplate) => {
    return factory.VariableStatement.create({
        declarationList: factory.VariableDeclarationList.create({
            declarations: [
                factory.VariableDeclaration.create({
                    name: "uri",
                    initializer: Utils.generateTemplateExpression(factory, urlTemplate),
                }),
            ],
            flag: "const",
        }),
    });
};
export const generateUrlTemplateExpression = (factory, requestUri, pathParameters) => {
    const patternMap = pathParameters.reduce((previous, item) => {
        return { ...previous, [`{${item.name}}`]: item.name };
    }, {});
    const urlTemplate = [];
    let temporaryStringList = [];
    // TODO generateVariableIdentifierに噛み合わ下げいいように変換する
    const replaceText = (text) => {
        let replacedText = text;
        Object.keys(patternMap).forEach(pathParameterName => {
            if (new RegExp(pathParameterName).test(replacedText)) {
                const { text, escaped } = escapeText(patternMap[pathParameterName]);
                const variableDeclareText = escaped ? `params.parameter[${text}]` : `params.parameter.${text}`;
                replacedText = replacedText.replace(new RegExp(pathParameterName, "g"), variableDeclareText);
            }
        });
        return replacedText === text ? undefined : replacedText;
    };
    const requestUrlTicks = Utils.multiSplitStringToArray(requestUri, Object.keys(patternMap));
    requestUrlTicks.forEach((requestUriTick, index) => {
        if (requestUri === "") {
            temporaryStringList.push("");
            return;
        }
        const replacedText = replaceText(requestUriTick);
        if (replacedText) {
            if (temporaryStringList.length > 0) {
                const value = temporaryStringList.join("/");
                urlTemplate.push({
                    type: "string",
                    value: value,
                });
                temporaryStringList = [];
            }
            urlTemplate.push({
                type: "property",
                value: Utils.generateVariableIdentifier(factory, replacedText),
            });
        }
        else {
            temporaryStringList.push(requestUriTick);
        }
        if (index === requestUrlTicks.length - 1) {
            const value = temporaryStringList.join("/");
            if (value === "") {
                urlTemplate.push({
                    type: "string",
                    value: requestUri.endsWith("/") ? "/" : "",
                });
            }
            else {
                urlTemplate.push({
                    type: "string",
                    value: value,
                });
            }
        }
    });
    return urlTemplate;
};
export const create = (factory, requestUri, pathParameters, methodType) => {
    const urlTemplate = pathParameters.length > 0 ? generateUrlTemplateExpression(factory, requestUri, pathParameters) : [{ type: "string", value: requestUri }];
    if (methodType === "currying-function") {
        return generateUriVariableStatement(factory, urlTemplate);
    }
    if (methodType === "class") {
        const variableExpression = factory.PropertyAccessExpression.create({
            name: "baseUrl",
            expression: "this",
        });
        return generateUrlVariableStatement(factory, urlTemplate, variableExpression);
    }
    if (methodType === "function") {
        const variableExpression = factory.Identifier.create({
            name: "_baseUrl",
        });
        return generateUrlVariableStatement(factory, urlTemplate, variableExpression);
    }
    throw new Error(`Invalid MethodType: ${methodType}`);
};
//# sourceMappingURL=PathParameter.js.map