import ts from "typescript";
import * as ModuleBlock from "./ModuleBlock";
import { generateComment } from "./utils";
// eslint-disable-next-line no-unused-vars
export const findStatement = (_context) => (params) => {
    let statement;
    params.node.forEachChild(node => {
        if (ts.isModuleDeclaration(node) && node.name.text === params.name) {
            statement = node;
        }
    });
    return statement;
};
export const create = ({ factory }) => (params) => {
    const node = factory.createModuleDeclaration(params.export && [factory.createModifier(ts.SyntaxKind.ExportKeyword)], factory.createIdentifier(params.name), factory.createModuleBlock(params.statements), ts.NodeFlags.Namespace);
    if (params.comment) {
        const comment = generateComment(params.comment, params.deprecated);
        return ts.addSyntheticLeadingComment(node, ts.SyntaxKind.MultiLineCommentTrivia, comment.value, comment.hasTrailingNewLine);
    }
    return node;
};
export const createMultiple = (context) => (params) => {
    const names = params.names.reverse();
    const firstName = names[0];
    const restNames = names.slice(1, names.length);
    const child = create(context)({
        export: true,
        name: firstName,
        statements: params.statements,
        comment: params.comment,
        deprecated: params.deprecated,
    });
    return restNames.reduce((previousStatement, currentName) => {
        return create(context)({
            export: true,
            name: currentName,
            statements: [previousStatement],
        });
    }, child);
};
export const update = (context) => (params) => {
    const { factory } = context;
    const { node, statements } = params;
    if (node.body && ts.isModuleBlock(node.body)) {
        const body = ModuleBlock.update(context)({ node: node.body, statements });
        return factory.updateModuleDeclaration(node, node.decorators, node.modifiers, node.name, body);
    }
    return factory.updateModuleDeclaration(node, node.decorators, node.modifiers, node.name, node.body);
};
export const addStatements = (context) => (params) => {
    const { factory } = context;
    const { node, statements } = params;
    if (node.body && ts.isModuleBlock(node.body)) {
        const body = ModuleBlock.update(context)({ node: node.body, statements: node.body.statements.concat(statements) });
        return factory.updateModuleDeclaration(node, node.decorators, node.modifiers, node.name, body);
    }
    return factory.updateModuleDeclaration(node, node.decorators, node.modifiers, node.name, node.body);
};
export const make = (context) => {
    return {
        findNamespace: findStatement(context),
        create: create(context),
        update: update(context),
        createMultiple: createMultiple(context),
        addStatements: addStatements(context),
    };
};
//# sourceMappingURL=Namespace.js.map