import ts from "typescript";
import type { Factory } from "../TsGenerator";
export interface FormatConversion {
    /**
     * Search parameters for the Schema to be converted
     */
    selector: {
        /**
         * DataType format
         *
         * @see https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.1.0.md#data-types
         */
        format: string;
    };
    /**
     * Output format
     */
    output: {
        /**
         * The type after conversion. The input string will be output as a typedef.
         */
        type: string[];
        /**
         * How to handle typedefs when there is more than one.
         *
         * Default oneOf
         */
        multiType?: "allOf" | "oneOf";
    };
}
/**
 * ユーザーが利用できる各種変換オプション
 */
export interface Options {
    formatConversions?: FormatConversion[];
}
export interface Types {
    /**
     * operationIdに対するescape
     */
    escapeOperationIdText: (operationId: string) => string;
    /**
     * interface/namespace/typeAliasのnameをescapeする
     * import/exportなどの予約語も裁く
     */
    escapeDeclarationText: (text: string) => string;
    /**
     * Schemas.A.B.Cに対するEscape
     */
    escapeReferenceDeclarationText: (text: string) => string;
    /**
     * 非破壊: PropertySignatureのname用のescape
     */
    escapePropertySignatureName: (text: string) => string;
    /**
     * 破壊: TypeReferenceのname用のescape
     */
    escapeTypeReferenceNodeName: (text: string) => string;
    generateResponseName: (operationId: string, statusCode: string) => string;
    generateArgumentParamsTypeDeclaration: (operationId: string) => string;
    generateRequestContentTypeName: (operationId: string) => string;
    generateResponseContentTypeName: (operationId: string) => string;
    generateParameterName: (operationId: string) => string;
    generateRequestBodyName: (operationId: string) => string;
    generateFunctionName: (operationId: string) => string;
    convertFormatTypeNode: (schema: {
        format?: string;
    }) => undefined | ts.TypeNode;
}
/**
 * ユーザーが利用できる各種変換オプション
 */
export declare const create: (factory: Factory.Type, options?: Options) => Types;
